@@ -1,17 +1,14 @@
-#ifndef FUNCPROTO_H
-#define FUNCPROTO_H
-
 #if __STDC__ || defined(__cplusplus)
 #  undef WANT_PROTOTYPES
 #endif
 
 #ifdef WANT_PROTOTYPES
-#  define DOPROTO
+#  define ELK_USE_PROTOTYPES
 #  define ELLIPSIS
 #endif
 
 #ifdef __cplusplus
-#  define DOPROTO
+#  define ELK_USE_PROTOTYPES
 #  define ELLIPSIS ...
 #  define C_LINKAGE_BEGIN extern "C" {
 #  define C_LINKAGE_END   }
@@ -21,21 +18,18 @@
 #endif
 
 #if __STDC__ && !defined(__cplusplus)
-#  define DOPROTO
+#  define ELK_USE_PROTOTYPES
 #  define ELLIPSIS
 #endif
 
 #ifdef NO_PROTOTYPES
-#  undef DOPROTO
+#  undef ELK_USE_PROTOTYPES
 #endif
 
-#ifdef DOPROTO
+#ifdef ELK_USE_PROTOTYPES
 #  define P_(args) args
 #else
 #  define P_(args) ()
 #  define ELLIPSIS
-#endif
-
-#undef DOPROTO
-
+#  define const
 #endif
