@@ -1,13 +1,6 @@
-#ifndef PARAM_H
-#define PARAM_H
+#define INITFILE             "initscheme.scm"
 
-#if defined(vax) || defined(__vax__)
-   typedef int jmp_buf[17];
-#else
-#  include <setjmp.h>
-#endif
-
-#define INITFILE             "initscheme"
+#define LOADPATH_ENV         "ELK_LOADPATH"  /* environ sym for loadpath */
 
 #define OBARRAY_SIZE         1009      /* symbol hash table size */
 
@@ -24,10 +17,6 @@
 #define DEF_PRINT_DEPTH      20
 #define DEF_PRINT_LEN        1000
 
-#define FLONUM_FORMAT        "%.15g"
-
 #ifdef CAN_DUMP
 #  define INITIAL_STK_OFFSET   (20*1024)       /* 2*NCARGS */
 #endif
-
-#endif
