@@ -1,16 +1,21 @@
 #include "xt.h"
 
-#define MAX_FUNCTIONS            512
-
+static max_functions = 512;
 static Object Functions;
 
 int Register_Function (x) Object x; {
     register i;
+    Object v;
 
-    for (i = 0; i < MAX_FUNCTIONS; i++)
+    for (i = 0; i < max_functions; i++)
 	if (Nullp (VECTOR(Functions)->data[i])) break;
-    if (i == MAX_FUNCTIONS)
-	Primitive_Error ("too many registered functions");
+    if (i == max_functions) {
+	max_functions *= 2;
+	v = Make_Vector (max_functions, Null);
+	bcopy ((char *)VECTOR(Functions)->data, (char *)VECTOR(v)->data,
+	    i * sizeof (Object));
+	Functions = v;
+    }
     VECTOR(Functions)->data[i] = x;
     return i;
 }
@@ -24,6 +29,6 @@ void Deregister_Function (i) int i; {
 }
 
 init_xt_function () {
-    Functions = Make_Vector (MAX_FUNCTIONS, Null);
+    Functions = Make_Vector (max_functions, Null);
     Global_GC_Link (Functions);
 }
