@@ -1,7 +1,7 @@
 #include <math.h>
 #include <ctype.h>
 
-#include "scheme.h"
+#include "kernel.h"
 
 Object Make_Uninitialized_Bignum (size) {
     Object big;
@@ -41,9 +41,9 @@ Object Copy_S_Bignum (s) struct S_Bignum *s; {
     return big;
 }
 
-Object Make_Bignum (buf, neg, radix) char *buf; {
+Object Make_Bignum (buf, neg, radix) const char *buf; {
     Object big;
-    register char *p;
+    register const char *p;
     register c;
     register size = (strlen (buf) + 4) / 4;
 
@@ -235,52 +235,98 @@ Object Bignum_To_String (x, radix) Object x; {
     return ret;
 }
 
-
 Bignum_To_Integer (x) Object x; {
-    unsigned n = 0;
-    int s = BIGNUM(x)->usize;
+    unsigned ret = 0;
+    int i, shift = 0, size = BIGNUM(x)->usize;
+    int digits = sizeof(int)/2;
 
-    if (s) {
-	n = BIGNUM(x)->data[0];
-	if (s > 1) {
-	    n |= BIGNUM(x)->data[1] << 16;
-	    if (s > 2)
-err:		
-		Primitive_Error ("integer out of range: ~s", x);
-	}
-    }
+    if (size > digits)
+err:
+	Primitive_Error ("integer out of range: ~s", x);
+    for (i = 0; i < digits && i < size; i++, shift += 16)
+	ret |= (unsigned)BIGNUM(x)->data[i] << shift;
     if (Truep (BIGNUM(x)->minusp)) {
-	if (n > (~(unsigned)0 >> 1) + 1)
+	if (ret > (~(unsigned)0 >> 1) + 1)
 	    goto err;
-	return -n;
+	return -ret;
     } else {
-	if (n > ~(unsigned)0 >> 1)
+	if (ret > ~(unsigned)0 >> 1)
 	    goto err;
-	return n;
+	return ret;
+    }
+}
+
+long Bignum_To_Long (x) Object x; {
+    unsigned long ret = 0;
+    int i, shift = 0, size = BIGNUM(x)->usize;
+    int digits = sizeof(long)/2;
+
+    if (size > digits)
+err:
+	Primitive_Error ("integer out of range: ~s", x);
+    for (i = 0; i < digits && i < size; i++, shift += 16)
+	ret |= (unsigned long)BIGNUM(x)->data[i] << shift;
+    if (Truep (BIGNUM(x)->minusp)) {
+	if (ret > (~(unsigned long)0 >> 1) + 1)
+	    goto err;
+	return -ret;
+    } else {
+	if (ret > ~(unsigned long)0 >> 1)
+	    goto err;
+	return ret;
     }
 }
 
 Object Integer_To_Bignum (i) {
-    Object big = Make_Uninitialized_Bignum (2);
+    int k, digits = sizeof(int)/2;
+    Object big = Make_Uninitialized_Bignum (digits);
     unsigned n = i;
 
     if (i < 0) {
 	BIGNUM(big)->minusp = True;
 	n = -i;
     }
-    BIGNUM(big)->data[0] = n;
-    BIGNUM(big)->data[1] = n >> 16;
-    BIGNUM(big)->usize = 2;
+    for (k = 0; k < digits; k++, n >>= 16)
+	BIGNUM(big)->data[k] = n & 0xffff;
+    BIGNUM(big)->usize = k;
     Bignum_Normalize_In_Place (BIGNUM(big));
     return big;
 }
 
 Object Unsigned_To_Bignum (i) unsigned i; {
-    Object big = Make_Uninitialized_Bignum (2);
+    int k, digits = sizeof(int)/2;
+    Object big = Make_Uninitialized_Bignum (digits);
 
-    BIGNUM(big)->data[0] = i;
-    BIGNUM(big)->data[1] = i >> 16;
-    BIGNUM(big)->usize = 2;
+    for (k = 0; k < digits; k++, i >>= 16)
+	BIGNUM(big)->data[k] = i & 0xffff;
+    BIGNUM(big)->usize = k;
+    Bignum_Normalize_In_Place (BIGNUM(big));
+    return big;
+}
+
+Object Long_To_Bignum (i) long i; {
+    int k, digits = sizeof(long)/2;
+    Object big = Make_Uninitialized_Bignum (digits);
+    unsigned long n = i;
+
+    if (i < 0) {
+	BIGNUM(big)->minusp = True;
+	n = -i;
+    }
+    for (k = 0; k < digits; k++, n >>= 16)
+	BIGNUM(big)->data[k] = n & 0xffff;
+    BIGNUM(big)->usize = k;
+    Bignum_Normalize_In_Place (BIGNUM(big));
+    return big;
+}
+
+Object Unsigned_Long_To_Bignum (i) unsigned long i; {
+    int k, digits = sizeof(long)/2;
+    Object big = Make_Uninitialized_Bignum (digits);
+
+    for (k = 0; k < digits; k++, i >>= 16)
+	BIGNUM(big)->data[k] = i & 0xffff;
+    BIGNUM(big)->usize = k;
     Bignum_Normalize_In_Place (BIGNUM(big));
     return big;
 }
@@ -486,7 +532,7 @@ Object Bignum_Fixnum_Multiply (x, y) Object x, y; {   /* bignum * fixnum */
     Object big;
     register size, xsize, i;
     register gran_t *xbuf, *zbuf;
-    int yn = FIXNUM(y);
+    int fix = FIXNUM(y);
     register unsigned yl, yh;
     GC_Node;
     
@@ -495,14 +541,14 @@ Object Bignum_Fixnum_Multiply (x, y) Object x, y; {   /* bignum * fixnum */
     size = xsize + 2;
     big = Make_Uninitialized_Bignum (size);
     BIGNUM(big)->usize = size;
-    if (Truep (BIGNUM(x)->minusp) != (yn < 0))
+    if (Truep (BIGNUM(x)->minusp) != (fix < 0))
 	BIGNUM(big)->minusp = True;
     bzero ((char *)BIGNUM(big)->data, size * sizeof (gran_t));
     xbuf = BIGNUM(x)->data;
-    if (yn < 0)
-	yn = -yn;
-    yl = yn & 0xFFFF;
-    yh = yn >> 16;
+    if (fix < 0)
+	fix = -fix;
+    yl = fix & 0xFFFF;
+    yh = fix >> 16;
     zbuf = BIGNUM(big)->data;
     for (i = 0; i < xsize; ++i) {
 	register unsigned xf = xbuf[i];
@@ -562,17 +608,17 @@ Object Bignum_Fixnum_Divide (x, y) Object x, y; {   /* bignum / fixnum */
     Object big;
     register xsize, i;
     register gran_t *xbuf, *zbuf;
-    int yn = FIXNUM(y);
+    int fix = FIXNUM(y);
     int xminusp, yminusp = 0;
     register unsigned rem;
     GC_Node;
 
     GC_Link (x);
-    if (yn < 0) {
-	yn = -yn;
+    if (fix < 0) {
+	fix = -fix;
 	yminusp = 1;
     }
-    if (yn > 0xFFFF) {
+    if (fix > 0xFFFF) {
 	big = Integer_To_Bignum (FIXNUM(y));
 	GC_Unlink;
 	return Bignum_Divide (x, big);
@@ -589,8 +635,8 @@ Object Bignum_Fixnum_Divide (x, y) Object x, y; {   /* bignum / fixnum */
     for (i = xsize; --i >= 0; ) {
 	rem <<= 16;
 	rem += xbuf[i];
-	zbuf[i] = rem / yn;
-	rem %= yn;
+	zbuf[i] = rem / fix;
+	rem %= fix;
     }
     GC_Unlink;
     Bignum_Normalize_In_Place (BIGNUM(big));
@@ -635,7 +681,7 @@ Object Bignum_Divide (x, y) Object x, y; {   /* bignum / bignum */
     bcopy ((char *)POINTER(y), (char *)dor, dorsize);
     dor->size = dorsize = BIGNUM(y)->usize;
 
-    scale = 65536 / (dor->data[dor->usize - 1] + 1);
+    scale = 65536 / (unsigned int)(dor->data[dor->usize - 1] + 1);
     Bignum_Mult_In_Place (dend, scale);
     if (dend->usize < dend->size)
 	dend->data[dend->usize++] = 0;
@@ -677,7 +723,7 @@ Object Bignum_Divide (x, y) Object x, y; {   /* bignum / bignum */
 	    k += *dep;
 	    k -= prod & 0xFFFF;
 	    *dep++ = k;
-	    k >>= 16;
+	    ASR(k, 16);
 	    k -= prod >> 16;
 	}
 	k += *dep;
@@ -688,7 +734,7 @@ Object Bignum_Divide (x, y) Object x, y; {   /* bignum / bignum */
 	    for (dop = dor->data, edop = dop + dor->usize; dop < edop; ) {
 		k += *dep + *dop++;
 		*dep++ = k;
-		k >>= 16;
+		ASR(k, 16);
 	    }
 	    k += *dep;
 	    *dep = k;
