@@ -1,4 +1,4 @@
-#include "scheme.h"
+#include "kernel.h"
 
 Object Const_Cons (car, cdr) Object car, cdr; {
     Object ret = P_Cons (car, cdr);
@@ -10,7 +10,7 @@ Object P_Cons (car, cdr) Object car, cdr; {
     Object cell;
     GC_Node2;
 
-#ifdef INCREMENTAL_GC
+#ifdef GENERATIONAL_GC
     GC_Link2 (car, cdr);
     cell = Alloc_Object (sizeof (struct S_Pair), T_Pair, 0);
     GC_Unlink;
@@ -214,7 +214,7 @@ Object P_Make_List (n, init) Object n, init; {
     Object list;
     GC_Node;
 
-    if ((len = Get_Integer (n)) < 0)
+    if ((len = Get_Exact_Integer (n)) < 0)
 	Range_Error (n);
     list = Null;
     GC_Link (init);
@@ -313,7 +313,8 @@ Object P_Reverse_Set (x) Object x; {
 Object P_List_Tail (x, num) Object x, num; {
     register n;
 
-    for (n = Get_Integer (num); n > 0 && !Nullp (x); n--, x = P_Cdr (x)) ;
+    for (n = Get_Exact_Integer (num); n > 0 && !Nullp (x); n--, x = P_Cdr (x))
+	;
     return x;
 }
 
