@@ -1,4 +1,4 @@
-#include "scheme.h"
+#include "kernel.h"
 
 /*ARGSUSED*/
 Dummy_Visit (p, fp) Object *p; int (*fp)(); {
@@ -13,7 +13,7 @@ TYPEDESCR Types[MAX_TYPE] = {
     { 1, "real",		0, 0, 0, 0, 0, 0, },
     { 0, "null",		0, 0, 0, 0, 0, 0, },
     { 0, "boolean",		0, 0, 0, 0, 0, 0, },
-    { 0, "void",		0, 0, 0, 0, 0, 0, },
+    { 0, "unused",              0, 0, 0, 0, 0, 0, }, /* was void */
     { 0, "unbound",		0, 0, 0, 0, 0, 0, },
     { 0, "special",		0, 0, 0, 0, 0, 0, },
     { 0, "character",		0, 0, 0, 0, 0, 0, },
@@ -37,30 +37,30 @@ Wrong_Type (x, t) Object x; register t; {
     Wrong_Type_Combination (x, Types[t].name);
 }
 
-Wrong_Type_Combination (x, name) Object x; register char *name; {
+Wrong_Type_Combination (x, name) Object x; register const char *name; {
     register t = TYPE(x);
-    register char *p;
+    register const char *p;
     char buf[100];
 
     if (t < 0 || t >= MAX_TYPE || !(p = Types[t].name))
-	Panic ("bad type");
+	Panic ("bad type1");
     sprintf (buf, "wrong argument type %s (expected %s)", p, name);
     Primitive_Error (buf);
 }
 
 Object P_Type (x) Object x; {
     register t = TYPE(x);
-    register char *p;
+    register const char *p;
 
     if (t < 0 || t >= MAX_TYPE || !(p = Types[t].name))
-	Panic ("bad type");
+	Panic ("bad type2");
     return Intern (p);
 }
 
 /* Not used by the interpreter kernel (lint may complain).
  */
 Define_Type (t, name, size, const_size, eqv, equal, print, visit) register t;
-	char *name;
+	const char *name;
 	int (*size)(), (*eqv)(), (*equal)(), (*print)(), (*visit)(); {
     register TYPEDESCR *p;
 
@@ -72,10 +72,11 @@ Define_Type (t, name, size, const_size, eqv, equal, print, visit) register t;
 	    Primitive_Error ("out of types");
     } else {
 	if (t < 0 || t >= MAX_TYPE)
-	    Primitive_Error ("bad type");
+	    Primitive_Error ("bad type in define-type");
 	if (Types[t].name)
 	    Primitive_Error ("type already in use");
     }
+    Disable_Interrupts;
     p = &Types[t];
     p->haspointer = 1;		/* Assumption */
     p->name = name;
@@ -85,9 +86,6 @@ Define_Type (t, name, size, const_size, eqv, equal, print, visit) register t;
     p->equal = equal;
     p->print = print;
     p->visit = visit;
+    Enable_Interrupts;
     return t;
 }
-
-Object P_Voidp (x) Object x; {  /* Don't know a better place for this. */
-    return TYPE(x) == T_Void ? True : False;
-}
